import { isFunction } from '../util'

const _events = Symbol('events')
const lookup = new Map()

export class Events {
  constructor() {
    this[_events] = {}
  }

  on(type, callback) {
    this[_events][type] = this[_events][type] || []
    this[_events][type].push(callback.bind(this))
  }

  once(type, fn) {
    const self = this

    const listener = function() {
      fn.call(this, arguments)
      self.off(type, listener)
    }

    return this.on(type, listener)
  }

  off(type, fn) {
    if (isFunction(fn)) {
      const listeners = this.getListeners(type)
      const idx = listeners.indexOf(fn)
      listeners.splice(idx, 1)
    } else {
      if (this[_events][type]) {
        this[_events][type] = []
      } else {
        this[_events] = Object.create(null)
      }
    }
  }

  listenTo(target, type, callback) {
    if (!lookup.get(target)) {
      lookup.set(target, Object.create(null))
    }

    lookup.get(target)[type] = lookup.get(target)[type] || []
    lookup.get(target)[type].push(callback.bind(this))
  }

  emit(...args) {
    const type = args[0]
    const payloads = args.slice(1)
    const ctx = this

    let handlers = []

    if (lookup.get(ctx)) {
      // 当前对象被监听
      handlers.push.apply(handlers, lookup.get(ctx)[type] || [])
    }

    handlers.push.apply(handlers, this[_events][type] || [])

    handlers.forEach(handler => handler.apply(this, payloads))
  }
}
