export function debounce(fn, wait = 0, immediate = false) {
  let context = null
  let args = null
  let timer = null
  let result = null
  let isCalled = false

  const later = () => {
    result = fn.apply(context, args)
  }

  return function() {
    context = this
    args = arguments

    clearTimeout(timer)

    if (immediate && !isCalled) {
      result = fn.apply(context, args)
      isCalled = true
    } else {
      timer = setTimeout(later, wait)
    }

    return result
  }
}
