export function throttle(
  fn,
  wait = 0,
  options = {
    leading: false,
    trailing: false
  }
) {
  let context = null
  let args = null
  let result = null
  let previous = 0
  let timer = null

  const later = () => {
    previous = options.leading ? Date.now() : 0
    timer = null
    result = fn.apply(context, args)
  }

  return function() {
    const now = Date.now()

    if (!previous && !options.leading) {
      // 非延迟开始前调用
      previous = now
    }

    const remaining = wait - (now - previous) // 剩余等待时间

    context = this
    args = arguments

    if (remaining <= 0 || remaining > wait) {
      // 调用
      if (timer) {
        clearTimeout(timer)
        timer = null
      }

      previous = now
      result = fn.apply(context, args)
    } else if (!timer && options.trailing) {
      // 延迟结束后调用
      timer = setTimeout(later, remaining)
    }

    return result
  }
}
